. ${PSScriptRoot}\def_upgrade.ps1

${active_mq_loc_final} = "${ACTIVE_MQ_DEFAULT}"
${java_loc_final} = "${JAVA_DEFAULT}"
${postgresql_loc_final} = "${POSTGRESQL_PATH_DEFAULT}"
${tomcat_loc_final} = "${TOMCAT_DEFAULT}"

${do_please} = Read-Host "Do you want to compare Tomcat versions? [Y/n]"
If ("${do_please}" -eq "" -Or "${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${tomcat_loc} = Read-Host "  Where is your Tomcat located? [${tomcat_loc_final}]"
    If ("${tomcat_loc}" -ne "") { ${tomcat_loc_final} = ${tomcat_loc} }
    ${result} = (Get-PathValidity ${tomcat_loc_final} ${TOMCAT_VALIDITY})
    If ($result -eq ${TRUE}) {
      ${input_invalid} = ${FALSE}
      ${tomcat_version_output} = (cmd /c "${tomcat_loc_final}\bin\setenv.bat & ${tomcat_loc_final}\bin\version.bat")
      ${tomcat_version_exist} = (${tomcat_version_output} |
        Select-String -Pattern 'Server version:') -replace "Server version:\s+Apache Tomcat/", ""
      Compare-VersionsWithResult "Tomcat" ${tomcat_version_exist} ${TOMCAT_VERSION_TARGET}

      ${java_loc_final} = (${tomcat_version_output} |
        Select-String -Pattern 'Using JRE_HOME:') -replace 'Using JRE_HOME:\s+"', '' -replace '"$', ''
      If ("${java_loc_final}" -ne "") {
        ${global:ALREADY_OBTAINED_JAVA_LOC} = ${TRUE}
        Write-Host "  Tomcat is using Java located at `"${java_loc_final}`""
      }
    }
  }
}


${do_please} = Read-Host "Do you want to compare ActiveMQ versions? [y/N]"
If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${active_mq_loc} = Read-Host "  Where is your ActiveMQ located? [${active_mq_loc_final}]"
    If ("${active_mq_loc}" -ne "") { ${active_mq_loc_final} = ${active_mq_loc} }
    ${result} = (Get-PathValidity ${active_mq_loc_final} ${ACTIVE_MQ_VALIDITY})
    If ($result -eq ${TRUE} ) {
      ${input_invalid} = ${FALSE}

      ${java_home_loc_activemq} = (Get-Content -Path "${active_mq_loc_final}\bin\activemq.bat" |
        Select-String -Pattern '.*set\sJAVA_HOME=' ) -replace ".*set\sJAVA_HOME=", "" -replace "`"", ""
      If ("${java_home_loc_activemq}" -ne "") {
        Write-Host "  ActiveMQ is using Java located at `"${java_home_loc_activemq}`""
        If (-not ${global:ALREADY_OBTAINED_JAVA_LOC}) {
          ${global:ALREADY_OBTAINED_JAVA_LOC} = ${TRUE}
          ${java_loc_final} = ${java_home_loc_activemq}
        }
      } Else {
        Get-ValidJavaPath
        ${global:JAVA_HOME} = ${java_loc_final}
      }
      ${active_mq_version_exist} = (cmd /c "${active_mq_loc_final}\bin\activemq --version" |
        Select-String -Pattern 'ActiveMQ\s') -replace "ActiveMQ\s+", ""
      Compare-VersionsWithResult "ActiveMQ" ${active_mq_version_exist} ${ACTIVE_MQ_VERSION_TARGET}
    }
  }
}


${do_please} = Read-Host "Do you want to compare Java versions? [Y/n]"
If ("${do_please}" -eq "" -Or "${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  If (-not ${global:ALREADY_OBTAINED_JAVA_LOC}) {
    Get-ValidJavaPath
  }
  ${java_version_exist} = ((cmd /c "${java_loc_final}\bin\java.exe -version" 2>&1) |
    Select-String -Pattern "^.*\sversion") -replace "^.*\sversion\s+`"", "" -replace "`"", ""
  Compare-VersionsWithResult "Java" ${java_version_exist} ${JAVA_VERSION_TARGET}
}


${do_please} = Read-Host "Do you want to compare PostgreSQL versions? [y/N]"
If ("${do_please}" -eq "y" -Or "${do_please}" -eq "Y") {
  ${input_invalid} = ${TRUE}
  while (${input_invalid}) {
    ${postgresql_loc} = Read-Host "  Where is your PostgreSQL located? [${postgresql_loc_final}]"
    If ("${postgresql_loc}" -ne "") { ${postgresql_loc_final} = ${postgresql_loc} }
    ${result} = (Get-PathValidity ${postgresql_loc_final} ${POSTGRESQL_VALIDITY})
    If ($result -eq ${TRUE} ) {
      ${input_invalid} = ${FALSE}
      ${postgresql_version_exist} = (cmd /c "${postgresql_loc_final}\bin\psql --version" |
        Select-String -Pattern 'psql\s+\(PostgreSQL\)\s') -replace "psql\s+\(PostgreSQL\)\s+", ""
      Compare-VersionsWithResult "PostgreSQL" ${postgresql_version_exist} ${POSTGRESQL_VERSION_TARGET}
    }
  }
}


# SIG # Begin signature block
# MIIdcwYJKoZIhvcNAQcCoIIdZDCCHWACAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCEyNBLy1XtjRhT
# bK6T39avKGH55hH8CahqiCh0S+7UA6CCGFwwggO3MIICn6ADAgECAhAM5+DlF9hG
# /o/lYPwb8DA5MA0GCSqGSIb3DQEBBQUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0wNjExMTAwMDAwMDBa
# Fw0zMTExMTAwMDAwMDBaMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNVBAMTG0RpZ2lD
# ZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCC
# AQoCggEBAK0OFc7kQ4BcsYfzt2D5cRKlrtwmlIiq9M71IDkoWGAM+IDaqRWVMmE8
# tbEohIqK3J8KDIMXeo+QrIrneVNcMYQq9g+YMjZ2zN7dPKii72r7IfJSYd+fINcf
# 4rHZ/hhk0hJbX/lYGDW8R82hNvlrf9SwOD7BG8OMM9nYLxj+KA+zp4PWw25EwGE1
# lhb+WZyLdm3X8aJLDSv/C3LanmDQjpA1xnhVhyChz+VtCshJfDGYM2wi6YfQMlqi
# uhOCEe05F52ZOnKh5vqk2dUXMXWuhX0irj8BRob2KHnIsdrkVxfEfhwOsLSSplaz
# vbKX7aqn8LfFqD+VFtD/oZbrCF8Yd08CAwEAAaNjMGEwDgYDVR0PAQH/BAQDAgGG
# MA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0OBBYEFEXroq/0ksuCMS1Ri6enIZ3zbcgP
# MB8GA1UdIwQYMBaAFEXroq/0ksuCMS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUA
# A4IBAQCiDrzf4u3w43JzemSUv/dyZtgy5EJ1Yq6H6/LV2d5Ws5/MzhQouQ2XYFwS
# TFjk0z2DSUVYlzVpGqhH6lbGeasS2GeBhN9/CTyU5rgmLCC9PbMoifdf/yLil4Qf
# 6WXvh+DfwWdJs13rsgkq6ybteL59PyvztyY1bV+JAbZJW58BBZurPSXBzLZ/wvFv
# hsb6ZGjrgS2U60K3+owe3WLxvlBnt2y98/Efaww2BxZ/N3ypW2168RJGYIPXJwS+
# S86XvsNnKmgR34DnDDNmvxMNFG7zfx9jEB76jRslbWyPpbdhAbHSoyahEHGdreLD
# +cOZUbcrBwjOLuZQsqf6CkUvovDyMIIE/jCCA+agAwIBAgIQDUJK4L46iP9gQCHO
# FADw3TANBgkqhkiG9w0BAQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGln
# aUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhE
# aWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgVGltZXN0YW1waW5nIENBMB4XDTIxMDEw
# MTAwMDAwMFoXDTMxMDEwNjAwMDAwMFowSDELMAkGA1UEBhMCVVMxFzAVBgNVBAoT
# DkRpZ2lDZXJ0LCBJbmMuMSAwHgYDVQQDExdEaWdpQ2VydCBUaW1lc3RhbXAgMjAy
# MTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAMLmYYRnxYr1DQikRcpj
# a1HXOhFCvQp1dU2UtAxQtSYQ/h3Ib5FrDJbnGlxI70Tlv5thzRWRYlq4/2cLnGP9
# NmqB+in43Stwhd4CGPN4bbx9+cdtCT2+anaH6Yq9+IRdHnbJ5MZ2djpT0dHTWjaP
# xqPhLxs6t2HWc+xObTOKfF1FLUuxUOZBOjdWhtyTI433UCXoZObd048vV7WHIOsO
# jizVI9r0TXhG4wODMSlKXAwxikqMiMX3MFr5FK8VX2xDSQn9JiNT9o1j6BqrW7Ed
# MMKbaYK02/xWVLwfoYervnpbCiAvSwnJlaeNsvrWY4tOpXIc7p96AXP4Gdb+DUmE
# vQECAwEAAaOCAbgwggG0MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEEGA1UdIAQ6MDgwNgYJYIZIAYb9bAcBMCkw
# JwYIKwYBBQUHAgEWG2h0dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAfBgNVHSME
# GDAWgBT0tuEgHf4prtLkYaWyoiWyyBc1bjAdBgNVHQ4EFgQUNkSGjqS6sGa+vCgt
# HUQ23eNqerwwcQYDVR0fBGowaDAyoDCgLoYsaHR0cDovL2NybDMuZGlnaWNlcnQu
# Y29tL3NoYTItYXNzdXJlZC10cy5jcmwwMqAwoC6GLGh0dHA6Ly9jcmw0LmRpZ2lj
# ZXJ0LmNvbS9zaGEyLWFzc3VyZWQtdHMuY3JsMIGFBggrBgEFBQcBAQR5MHcwJAYI
# KwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBPBggrBgEFBQcwAoZD
# aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0U0hBMkFzc3VyZWRJ
# RFRpbWVzdGFtcGluZ0NBLmNydDANBgkqhkiG9w0BAQsFAAOCAQEASBzctemaI7zn
# GucgDo5nRv1CclF0CiNHo6uS0iXEcFm+FKDlJ4GlTRQVGQd58NEEw4bZO73+RAJm
# Te1ppA/2uHDPYuj1UUp4eTZ6J7fz51Kfk6ftQ55757TdQSKJ+4eiRgNO/PT+t2R3
# Y18jUmmDgvoaU+2QzI2hF3MN9PNlOXBL85zWenvaDLw9MtAby/Vh/HUIAHa8gQ74
# wOFcz8QRcucbZEnYIpp1FUL1LTI4gdr0YKK6tFL7XOBhJCVPst/JKahzQ1HavWPW
# H1ub9y4bTxMd90oNcX6Xt/Q/hOvB46NJofrOp79Wz7pZdmGJX36ntI5nePk2mOHL
# KNpbh6aKLzCCBTAwggQYoAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcN
# AQELBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcG
# A1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJl
# ZCBJRCBSb290IENBMB4XDTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjEL
# MAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3
# LmRpZ2ljZXJ0LmNvbTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElE
# IENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEB
# APjTsxx/DhGvZ3cH0wsxSRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5
# idQ3Gde2qvCchqXYJawOeSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQ
# dLEoJrskacLCUvIUZ4qJRdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+
# rHCiq85/6XzLkqHlOzEcz+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE
# 86xnTrXE94zRICUj6whkPlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pz
# INAPZHM8np+mM6n9Gd8lk9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8C
# AQAwDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUF
# BwEBBG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMG
# CCsGAQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRB
# c3N1cmVkSURSb290Q0EuY3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3Js
# NC5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2
# hjRodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290
# Q0EuY3JsME8GA1UdIARIMEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxo
# dHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQW
# BBRaxLl7KgqjpepxA8Bg+S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYun
# pyGd823IDzANBgkqhkiG9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwt
# OhrE7zBh134LYP3DPQ/Er4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GO
# Yw4TS63XX0R58zYUBor3nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEo
# AjwukaPAJRHinBRHoXpoaK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhC
# MrI2iiQC/i9yfhzXSUWW6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOB
# m0/GkxAG/AeB+ova+YJJ92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zCCBTEw
# ggQZoAMCAQICEAqhJdbWMht+QeQF2jaXwhUwDQYJKoZIhvcNAQELBQAwZTELMAkG
# A1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRp
# Z2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENB
# MB4XDTE2MDEwNzEyMDAwMFoXDTMxMDEwNzEyMDAwMFowcjELMAkGA1UEBhMCVVMx
# FTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNv
# bTExMC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIFRpbWVzdGFtcGlu
# ZyBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAL3QMu5LzY9/3am6
# gpnFOVQoV7YjSsQOB0UzURB90Pl9TWh+57ag9I2ziOSXv2MhkJi/E7xX08PhfgjW
# ahQAOPcuHjvuzKb2Mln+X2U/4Jvr40ZHBhpVfgsnfsCi9aDg3iI/Dv9+lfvzo7oi
# PhisEeTwmQNtO4V8CdPuXciaC1TjqAlxa+DPIhAPdc9xck4Krd9AOly3UeGheRTG
# TSQjMF287DxgaqwvB8z98OpH2YhQXv1mblZhJymJhFHmgudGUP2UKiyn5HU+upgP
# hH+fMRTWrdXyZMt7HgXQhBlyF/EXBu89zdZN7wZC/aJTKk+FHcQdPK/P2qwQ9d2s
# rOlW/5MCAwEAAaOCAc4wggHKMB0GA1UdDgQWBBT0tuEgHf4prtLkYaWyoiWyyBc1
# bjAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzASBgNVHRMBAf8ECDAG
# AQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDB5Bggr
# BgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNv
# bTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lD
# ZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHoweDA6oDigNoY0aHR0cDov
# L2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6
# oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElE
# Um9vdENBLmNybDBQBgNVHSAESTBHMDgGCmCGSAGG/WwAAgQwKjAoBggrBgEFBQcC
# ARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzALBglghkgBhv1sBwEwDQYJ
# KoZIhvcNAQELBQADggEBAHGVEulRh1Zpze/d2nyqY3qzeM8GN0CE70uEv8rPAwL9
# xafDDiBCLK938ysfDCFaKrcFNB1qrpn4J6JmvwmqYN92pDqTD/iy0dh8GWLoXoIl
# HsS6HHssIeLWWywUNUMEaLLbdQLgcseY1jxk5R9IEBhfiThhTWJGJIdjjJFSLK8p
# ieV4H9YLFKWA1xJHcLN11ZOFk362kmf7U2GJqPVrlsD0WGkNfMgBsbkodbeZY4Ui
# jGHKeZR+WfyMD+NvtQEmtmyl7odRIeRYYJu6DC0rbaLEfrvEJStHAgh8Sa4TtuF8
# QkIoxhhWz0E0tmZdtnR79VYzIi8iNrJLokqV2PWmjlIwggUyMIIEGqADAgECAhAB
# Pe4iVgqMX6vbiX1xPVYPMA0GCSqGSIb3DQEBCwUAMHIxCzAJBgNVBAYTAlVTMRUw
# EwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20x
# MTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcg
# Q0EwHhcNMTgwNjExMDAwMDAwWhcNMjEwODIxMTIwMDAwWjBvMQswCQYDVQQGEwJH
# QjEQMA4GA1UEBxMHTmV3YnVyeTEmMCQGA1UEChMdTWljcm8gRm9jdXMgSW50ZXJu
# YXRpb25hbCBwbGMxJjAkBgNVBAMTHU1pY3JvIEZvY3VzIEludGVybmF0aW9uYWwg
# cGxjMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA2Vb7QjccsV83UO3b
# YCw/nTB6Bl3tGX0eSCCPsBtPTspWKz2GXWK7/1KeDUNPLnNBT1InLwNcaDxiX1TG
# SC5+gBfJiaTWB03ll7FpDbayyQ532smNvlQop2v5rgDkXQLu4Jg1JE3OLrppxBfP
# tBsjHw76kCqePA8cvTMWxUc6b7loX26dQe8v7+SeOI0qZ0tXCNwtgTu2513frUxj
# d/fY2F/toc9OjateJZQ5v5+WV7dOG/89bkvMAEHXegLcPZw7PT4aZP/ZVOYtoflS
# g6oT0m0N1LdsjyTEDbqmRm6/dt5sI9akmdln7hpOp1y9EMCgkGiNAwf9u4Lw2iUj
# NvoT5QIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1
# DlgwHQYDVR0OBBYEFLyS6yL4fTRV+hftOctsNinkVorLMA4GA1UdDwEB/wQEAwIH
# gDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8v
# Y3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYv
# aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmww
# TAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93
# d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAC
# hkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJl
# ZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsF
# AAOCAQEAZllSiMymdGRhqGhTP8p8Xdjmu1IZu6wg09n7oKLaH7qg3TfjqX+jjaQ2
# CTTyr2cwblYPa1Dp2sOqjGUNyrUvg22cfaSJAtbTNkpb8VtU8nEMjc3IaTdAw0hM
# +qHgYUaX1he6HrP8x51xJW67Z+Q6junwCPjo4fhEonJ4D9PBqQpRzpHtHy3D9rI6
# sgTSX3uWHqUet4Ehq49dSb2GZB8qAweUeM8ntCsEe2Trrei+HbrWn2F6DjpFCRA8
# VjhNvIsqOyltdsGT1HOGHz3hIi2G1Xr/T43sBmeqndopqavoj9GZsUMVp6RPRohn
# yGDTxINU2+Cs981b0UUjO5BQkkQfrzGCBG0wggRpAgEBMIGGMHIxCzAJBgNVBAYT
# AlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2Vy
# dC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNp
# Z25pbmcgQ0ECEAE97iJWCoxfq9uJfXE9Vg8wDQYJYIZIAWUDBAIBBQCggYQwGAYK
# KwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIB
# BDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQg
# bwqbS2lqtExVocNEahkV7YdKURyP4J1hBol+ickO2McwDQYJKoZIhvcNAQEBBQAE
# ggEAd/Kg6BX82aLWErcEgXpeUhzz6M6+Z9AgDlJE82Da1wrTEEYYP4gMQK08dzz0
# ZP/CbWexo2IC3gNIVyinBUSEhUJzfUfaeASV2YJQU3PmUbAqePyR+v05gziDAsNZ
# kkF3r+v49yq3zGJvdwZ6d2ImTLaDRCo1KF8eom0m54f7kg61RZdaehNQYXYJAaxB
# GucfM/IOi9HZf9kx+JHYxX1CxfgO7RJO9csesWn0e+Dqe2ZLfXtBxGc8Dc88IMSV
# hh6lO15uoBVYLH0ksEay/1DRZWDFDGYXoe4t30tovkgV0lIWrt2tHrl5tiu8etKS
# 0ptysK143QzSmGbVksCfTVoDGKGCAjAwggIsBgkqhkiG9w0BCQYxggIdMIICGQIB
# ATCBhjByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgVGltZXN0YW1waW5nIENBAhANQkrgvjqI/2BAIc4UAPDdMA0GCWCG
# SAFlAwQCAQUAoGkwGAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0B
# CQUxDxcNMjEwMzAyMTgxMjUwWjAvBgkqhkiG9w0BCQQxIgQgOs3AajsKfzZf4R6y
# IPo0N3EA0OcoYIhwX5geFVVPxR8wDQYJKoZIhvcNAQEBBQAEggEAcuu30bhaCYDF
# uU/HZ1a1YbjoyN+48YkzBAx20RZZz4yxRUrsppJPOdxKF4w5/cKyMyS1yNZkjUVc
# KdaWb4s73NiXxBgd++tXIMyX71A3X6uoF+gID4e3bsMcQcG6DrId51QhObGcx/zB
# MTw4yxqe764NJcaBXbuP1rMEnZo4VH1oxL2cGMjLO49DfOhMH+YBnJ1fYDLj2tG/
# iXGJxJf1J40Dx1A+R9Gf0k6PbdV5fQvU8xhoNr0iTKTod0zTycJizWsjQRZiaNcw
# 2XsL/82HEbVJSOrXmtQ6qCbE6t/bEaKL0Sk1sHol+MmBK4xBfwB9ZOvdH4NqDyZ6
# BnloRmkXZw==
# SIG # End signature block
